from autoppa.exception import FileProcessingError


class ReleaseSpecificInclude(object):
    """
    Processor includes all lines by default.  If a line starts with an
    C{AUTOPPA_INCLUDE($releases):} symbol where $releases is a
    comma-separated list of Ubuntu releases, the line will only be
    included if the release being built is in the list.  In other
    words, a line that starts with C{AUTOPPA_INCLUDE(dapper,edgy):}
    will only be included if the current release is either C{dapper}
    or C{edgy}.
    """

    def __init__(self, release):
        self.release = release

    PREFIX = "AUTOPPA_INCLUDE("
    SUFFIX = "):"

    def process(self, line, filename="<unknown>", line_number=0):
        if line.startswith(self.PREFIX):
            end = line.find(self.SUFFIX)
            if end == -1:
                raise FileProcessingError("""\
Missing closing parenthesis ')' in AUTOPPA_INCLUDE() symbol at
column %s in %s:%s:
%s
""" % (end+len(self.SUFFIX), filename, line_number, line))
            releases = line[len(self.PREFIX):end]
            releases = [release.strip() for release in releases.split(",")]
            if self.release not in releases:
                return None
            line = line[end+len(self.SUFFIX):]
        return line


class VersionReplacement(object):
    """
    Processor replaces C{AUTOPPA_VERSION($sample)} symbols in lines
    with C{AUTOPPA_VERSION($version)} where C{$sample} is, optionally,
    a sample version and C{$version} is the version for the build.  In
    other words, if C{AUTOPPA_VERSION()} is encountered, and the
    specified version is 0.10.11, the resulting output will be
    C{AUTOPPA_VERSION(0.10.11)}.

    This arrangement allows the following idiom to be using in Python
    source files::

        VERSION = 'AUTOPPA_VERSION(devel)'[len('AUTOPPA_VERSION('):-1]
    """

    def __init__(self, version):
        self.version = version

    PREFIX = "AUTOPPA_VERSION("
    SUFFIX = ")"

    def process(self, line, filename="<unknown>", line_number=0):
        begin = line.find(self.PREFIX)
        if begin != -1:
            end = line.find(self.SUFFIX, begin+len(self.PREFIX))
            if end == -1:
                raise FileProcessingError("""\
Missing closing parenthesis in AUTOPPA_VERSION() symbol at
column %s in %s:%s:
%s
""" % (begin, filename, line_number, line))
            symbol = line[begin:end+len(self.SUFFIX)]
            version = "AUTOPPA_VERSION(%s)" % self.version
            line = line.replace(symbol, version)
        return line
